// A fan game by poret. This game is not affiliated with SNK. This is a fan game which is not being sold or distributed for profit.

#pragma once

#include "CoreMinimal.h"
#include "PaperCharacter.h"
#include "KOFBaseCharacter.generated.h"

class UTextRenderComponent;

/**
 * This class is the default character for SideScroller2D, and it is responsible for all
 * physical interaction between the player and the world.
 *
 * The capsule component (inherited from ACharacter) handles collision with the world
 * The CharacterMovementComponent (inherited from ACharacter) handles movement of the collision capsule
 * The Sprite component (inherited from APaperCharacter) handles the visuals
 */
UCLASS()
class KOFFOREVER_API AKOFBaseCharacter : public APaperCharacter
{
	GENERATED_BODY()

	UTextRenderComponent* TextComponent;
	virtual void Tick(float DeltaSeconds) override;

	virtual void BeginPlay() override;

	virtual void PostInitializeComponents() override;
protected:
	/**
	 *	Secondary sprite that is mirrored across the ground axis and set to black to give the effect of a
	 *  character's shadow
	 */
	UPROPERTY(Category = Character, VisibleAnywhere, BlueprintReadOnly, meta = (AllowPrivateAccess = "true"))
	class UPaperFlipbookComponent* Shadow;
	
	// The animation to play while running around
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category=Animations)
	class UPaperFlipbook* RunningAnimation;

	// The animation to play while running around
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category=Animations)
	class UPaperFlipbook* WalkFwdAnimation;

	// The animation to play while running around
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category=Animations)
	class UPaperFlipbook* WalkBackAnimation;
	
	// The animation to play while idle (standing still)
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = Animations)
	class UPaperFlipbook* IdleAnimation;

	UPROPERTY(BlueprintReadOnly)
	bool bIsPossesedByPlayer1;
	
	/** Called to choose the correct animation to play based on the character's movement state */
	void UpdateAnimation();

	/** Called for side to side input */
	void MoveRight(float Value);

	void UpdateCharacter();
	
	// APawn interface
	virtual void SetupPlayerInputComponent(class UInputComponent* InputComponent) override;
	// End of APawn interface

public:
	AKOFBaseCharacter(const FObjectInitializer& ObjectInitializer);

	FORCEINLINE class UPaperFlipbookComponent* GetShadow() const { return Shadow; }

	/**
	 *	Returns whether or not this player is possessed by player 1
	 *
	 *	@returns true is possessed by player 1, false if possessed by player 2
	 */
	FORCEINLINE bool IsPossessedByPlayer1() const { return bIsPossesedByPlayer1; }

	/**
	 *	Sets which player this character is possesed by.
	 *
	 *	@param bIsPlayer1 true is possessed by player 1, false if possessed by player 2
	 */
	FORCEINLINE void SetIsPossesedByPlayer1(bool bIsPlayer1) { bIsPossesedByPlayer1 = bIsPlayer1; }
};
